<?php
// $Id$


/**
 * Service callback - add a new relationship.
 *
 * @param integer $source_nid
 * @param integer $target_nid
 * @param string $relationship_rid
 * @return boolean
 */
function graphmind_relationship_on_add_relationship($source_nid, $target_nid, $relationship_rid = NULL) {
  return graphmind_relationship_add_relationship($source_nid, $target_nid, $relationship_rid);
}


/**
 * Service callback - delete a relationship.
 *
 * @param integer $source_nid
 * @param integer $target_nid
 * @param string $relationship_rid
 * @return boolean
 */
function graphmind_relationship_on_delete_relationship($source_nid, $target_nid, $relationship_rid = NULL) {
  return graphmind_relationship_delete_relationship($source_nid, $target_nid, $relationship_rid);
}


/**
 * Service callback - get a nodes relationships.
 *
 * @param integer $source_nid
 * @param string $relationship_rid
 * @return array
 */
function graphmind_relationship_on_get_relationships($source_nid, $relationship_rid = NULL) {
  return graphmind_relationship_get_relationships($source_nid, $relationship_rid);
}


/**
 * Services callback - get a nodes subtree.
 *
 * @param integer $nid
 * @param integer $depth
 * @return array
 */
function graphmind_relationship_get_subtree($nid, $depth = null) {
  if ($depth == null) {
    $depth = GRAPHMIND_RELATIONSHIP_DEFAULT_DEPTH;
  }

  return _graphmind_relationship_get_subtree($nid, $depth);
}


/**
 * Build a subtree array to a node by it's relationships.
 *
 * @param integer $nid
 * @param integer $depth
 * @return array
 */
function _graphmind_relationship_get_subtree($nid, $depth) {
  if ($depth <= 0) return array();

  $res = db_query('
    SELECT *
    FROM {graphmind_relationship}
    WHERE source_nid = :nid
  ', array(':nid' => $nid));

  $relationships = array();
  foreach ($res as $row) {
    $row->relationships = _graphmind_relationship_get_subtree($row->target_nid, $depth - 1);
    $row->node = node_load($row->target_nid);

    if ($row->node) {
      $relationships[] = $row;
    }
  }

  return $relationships;
}


/**
 * Services callback - check if update needs.
 *
 * @param array $tree
 * @return boolean
 */
function graphmind_relationship_check_update($tree) {
  return _graphmind_relationship_check_subtree_update($tree);
}


/**
 * Check 1 level of a subtree if any update happened.
 *
 * @param array $subtree
 * @return boolean
 */
function _graphmind_relationship_check_subtree_update($subtree) {
  foreach ($subtree as $nid => $children) {
    $child_array = array();
    foreach ($children as $child) {
      $child_array = array_merge($child_array, array_keys((array)$child));
    }

    if (!empty($child_array)) {
      $existing_count = db_query('
        SELECT COUNT(*) AS count
        FROM {graphmind_relationship}
        WHERE source_nid = :nid AND target_nid IN (:children)
      ', array(':nid' => $nid, ':children' => $child_array))->fetchObject();
      $new_count = db_query('
        SELECT COUNT(*) AS count
        FROM {graphmind_relationship}
        WHERE source_nid = :nid AND target_nid NOT IN (:children)
      ', array(':nid' => $nid, ':children' => $child_array))->fetchObject();
      if (count($child_array) != $existing_count->count || $new_count->count > 0) {
        return FALSE;
      }
    }

    foreach ($children as $child) {
      $res = _graphmind_relationship_check_subtree_update($child);
      if (!$res) {
        return FALSE;
      }
    }
  }

  return TRUE;
}


/**
 * Service callback - get user color.
 *
 * @param integer $uid
 */
function graphmind_relationship_get_user_color($uid) {
  if (!isset($_SESSION['GraphmindRelationship']['UserColor'][$uid])) {
    $digits = array('8', '9', 'A', 'B', 'C', 'D', 'E', 'F');
    $length = count($digits);
    $r = $digits[rand(0, $length - 1)];
    $g = $digits[rand(0, $length - 1)];
    $b = $digits[rand(0, $length - 1)];
    $_SESSION['GraphmindRelationship']['UserColor'][$uid] = '0x' . $r . $r . $g . $g . $b . $b;
  }

  return $_SESSION['GraphmindRelationship']['UserColor'][$uid];
}
